//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998,1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#ifndef PACKER_H
#define PACKER_H

//#include "myBox.h"
#include <InterfaceKit.h>

enum {
	K_NONE =			0,
	// ---
	K_WARP_TOP =		1,
	K_WARP_BOTTOM =		2,
	// ---
	K_WARP_LEFT =		4,
	K_WARP_RIGHT =		8,
	// ---
	K_WARP_ALL =		16,
	// ---
	K_FILL_X =			32,
	K_FILL_Y =			64,
};

enum {
	K_TOP =			1,
	K_BOTTOM =		2,
	K_LEFT =		4,
	K_RIGHT =		8,
	K_FULL =                          16,
};

const uint32 K_FILL_BOTH = K_FILL_X | K_FILL_Y;
const uint32 K_FILL_NONE = K_NONE;

/* store widget placement info */
struct pack_info {

  pack_info(){};

  pack_info(BView *w,uint32 s,uint32 a,uint32 f,bool x,float hb,float vb
	    )
    {
      view=w;side=s;anchor=a;fill=f;expand=x;hborder=hb;vborder=vb;
    };
  
  // "override" widget bounds width&height to take border in account
  float Width() {return view->Bounds().Width()+hborder+hborder;};
  float Height() {return view->Bounds().Height()+vborder+vborder;};

  BView *view;// the widget
  uint32 side;// the side where to allocate the widget parcel 
  uint32 anchor;// where to anchor the widget in its parcel
  uint32 fill;// how the widget should fill its parcel
  bool expand;// deprecated ?
  float hborder,vborder;// horizontal & vertical border to add around the widget
}; 


class KPackView;

class KPack {
public:
	KPack(BView *view,
		uint32 side=K_TOP,
		uint32 anchor=K_TOP | K_LEFT,
		uint32 fill=K_FILL_NONE,
		BPoint border=BPoint(0,0)
		);
		
	KPack(struct pack_info info);
		
private:
	friend KPackView;

	struct pack_info fInfo;
};

class KPackView : public BBox {
public:
	KPackView(BRect frame, const char *name, 
		//bool propagate=false,
		uint32 resizingMode=B_FOLLOW_ALL_SIDES, 
		uint32 flags=0,
		border_style border = B_NO_BORDER);
		
	KPackView(const char *name,KPack **pack_list);
		
	virtual ~KPackView();
	
	virtual void PackChild(BView *view,
		uint32 side=K_TOP,
		uint32 anchor=K_TOP | K_LEFT,
		uint32 fill=K_FILL_NONE,
		//bool expand=false,
		BPoint border=BPoint(0,0)
		);
		
	void PackChild(KPack *pack);
		
	void AttachedToWindow(void); 
	virtual void AttachedToWindow(bool first_time); 
	
	//virtual void FrameResized(float width, float height);
	
private:
/*
	virtual void really_pack_child(BView *view,
		uint32 side=K_TOP,
		uint32 anchor=K_TOP | K_LEFT,
		uint32 fill=K_FILL_NONE,
		bool expand=false,
		float border=0
		);
*/
	virtual void really_pack_child(pack_info *info);
		
	void ExtendBy(float x,float y);
	BRect ComputeFreeParcel();
	
private:
	typedef BBox inherited;

	bool	fPropagate;
	bool	fAttachedOnce;
	
	BList	*fPackQueue,*fPackStack;

	BView	*fPackTop,*fPackBottom;
	BRect	fFree,fBounds;
	bool	fFull;
	int		fBorder;
};

#endif
